@@ -1,8 +1,7 @@
 """Main module of this project."""
 
-from .database import DATABASE_42
-from .expr_utils import build_node
-import random
+from .expr_utils import Node, build_node
+from .problem_utils import Problem
 import datetime
 
 
@@ -13,7 +12,7 @@ class FTPtsGame(object):
     Available methods (+ means playing, - means not playing):
     __init__(): initialization. (Entry point)
     is_playing(): show the status of current game. (+-)
-    generate_problem(): generate a problem from database or on custom. (-)
+    generate_problem(): generate a problem manually. (-)
     get_elapsed_time(): get the time elapsed during the game. (+)
     get_current_problem(): get current problem (tuple). (+)
     get_current_solutions(): get current solutions (list). (+)
@@ -24,12 +23,13 @@ class FTPtsGame(object):
     solve(): put forward a solution and show solution intervals. (+)
     """
 
-    def __init__(self):
+    def __init__(self, target=42):
         """Start the game session, serving as an initialization."""
         self.__valid = []  # this list stores readable answers
         self.__formula = []  # this list stores converted formulas
         self.__players = []  # this list stores player statistics
         self.__playing = False  # this stores playing status
+        self.__target = target
 
     def __status_check(self, required_status: bool = True):
         """A status checker."""
@@ -37,7 +37,7 @@ class FTPtsGame(object):
             raise PermissionError('Required status: %s' % required_status)
 
     def is_playing(self) -> bool:
-        """Incicate the game is started or not."""
+        """Indicate the game is started or not."""
         return self.__playing
 
     def get_elapsed_time(self) -> datetime.timedelta:
@@ -46,55 +46,14 @@ class FTPtsGame(object):
         elapsed = datetime.datetime.now() - self.__timer
         return elapsed
 
-    def __generate_problem_from_database(self, **kwargs) -> tuple:
-        """Generate a problem from database."""
-        minimum_solutions = kwargs[
-            'minimum_solutions'] if 'minimum_solutions' in kwargs else 1
-        maximum_solutions = kwargs[
-            'maximum_solutions'] if 'maximum_solutions' in kwargs else 100
-        problem_list = [
-            k for k in DATABASE_42.keys()
-            if minimum_solutions <= DATABASE_42[k] <= maximum_solutions
-        ]
-        return random.choice(problem_list)
-
-    def __generate_problem_by_user(self, **kwargs) -> tuple:
-        """Generate a problem by user."""
-        if 'problem' not in kwargs:
-            return self.__generate_problem_from_database()
-
-        problem = tuple(sorted(list(kwargs['problem'])))
-        if problem not in DATABASE_42:
-            raise ValueError('Problem has no solutions.')
-        return problem
-
-    def __generate_problem_by_probability(self, **kwargs) -> tuple:
-        """Generate a problem by frequency / probability."""
-        if 'prob' not in kwargs:
-            return self.__generate_problem_from_database()
-
-        prob_list = list(kwargs['prob'])
-        if len(prob_list) != len(DATABASE_42.keys()):
-            raise ValueError('Prob_list must match the size of the database.')
-
-        r = random.random() * sum(prob_list)
-        cumulative_prob = 0.0
-        for problem, problem_prob in zip(DATABASE_42.keys(), prob_list):
-            cumulative_prob += problem_prob
-            if r < cumulative_prob:
-                return problem
-
-    def generate_problem(self, method: str, **kwargs) -> tuple:
-        """Generate a random problem from the database."""
+    def generate_problem(self, problem):
+        """Generate a problem manually."""
         self.__status_check(required_status=False)
-        if method == 'database':
-            self.__problem = self.__generate_problem_from_database(**kwargs)
-        elif method == 'custom':
-            self.__problem = self.__generate_problem_by_user(**kwargs)
-        elif method == 'probability':
-            self.__problem = self.__generate_problem_by_probability(**kwargs)
-        else:
-            raise TypeError('Invalid problem type.')
+        self.__problem = tuple(sorted(problem))
+        self.__problem_class = Problem(list(self.__problem))
+        self.__problem_class.generate_answers(self.__target)
+        if len(self.__problem_class.distinct_answer_table) == 0:
+            raise ValueError('No solution found.')
 
     def get_current_problem(self) -> tuple:
         """Get current problem. Effective when playing."""
@@ -114,18 +73,36 @@ class FTPtsGame(object):
     def get_total_solution_number(self) -> int:
         """Get the number of total solutions. Effective when playing."""
         self.__status_check(required_status=True)
-        return DATABASE_42[self.__problem]
+        return len(self.__problem_class.distinct_answer_table)
 
-    def get_current_player_statistics(self) -> dict:
+    def get_remaining_solutions(self) -> list:
+        """Get remaining solutions. Effective when playing."""
+        self.__status_check(required_status=True)
+        current_solution_set = set()
+        for expr_str in self.__valid:
+            node = build_node(expr_str)
+            current_solution_set.add(
+                self.__problem_class.equivalence_dict[node.unique_id()])
+        return_list = []
+        for expr in self.__problem_class.distinct_answer_table:
+            if self.__problem_class.equivalence_dict[
+                    expr.unique_id()] not in current_solution_set:
+                return_list.append(str(expr))
+        return return_list
+
+    def get_current_player_statistics(self) -> list:
         """Get current player statistics. Effective when playing."""
         self.__status_check(required_status=True)
         return self.__players
 
-    def __validate_repeated(self, node: str):
+    def __validate_repeated(self, node: Node):
         """Validate distinguishing expressions. Private method."""
+        class_id = self.__problem_class.equivalence_dict[node.unique_id()]
         for ind in range(0, len(self.__formula)):
             cmp_node = self.__formula[ind]
-            if cmp_node == node or cmp_node.simplify() == node.simplify():
+            cmp_class_id = self.__problem_class.equivalence_dict[
+                cmp_node.unique_id()]
+            if cmp_class_id == class_id:
                 raise LookupError(self.__valid[ind])
 
     def solve(self, math_expr: str, player_id: int = -1) -> datetime.timedelta:
@@ -145,10 +122,9 @@ class FTPtsGame(object):
         node = build_node(math_expr)
         math_expr_value = node.evaluate()
         user_input_numbers = node.extract()
-        if math_expr_value != 42:
+        if math_expr_value != self.__target:
             raise ArithmeticError(str(math_expr_value))
         if tuple(sorted(user_input_numbers)) != self.__problem:
-            print(user_input_numbers)
             raise ValueError('Unmatched input numbers.')
 
         self.__validate_repeated(node)
